package javax.swing.plaf.synth;

import java.beans.PropertyChangeEvent;
import java.awt.Graphics;
import javax.swing.JPanel;
import javax.swing.plaf.ComponentUI;
import javax.swing.JComponent;
import java.beans.PropertyChangeListener;
import javax.swing.plaf.basic.BasicPanelUI;

public class SynthPanelUI extends BasicPanelUI implements PropertyChangeListener, SynthUI
{
    private SynthStyle style;
    
    public SynthPanelUI() {
    }
    
    public static ComponentUI createUI(JComponent component) {
        return new SynthPanelUI();
    }
    
    @Override
    public void installUI(JComponent c) {
        JPanel panel = (JPanel)c;
        super.installUI(c);
        this.installListeners(panel);
    }
    
    @Override
    public void uninstallUI(JComponent c) {
        this.uninstallListeners((JPanel)c);
        super.uninstallUI(c);
    }
    
    protected void installListeners(JPanel panel) {
        panel.addPropertyChangeListener(this);
    }
    
    protected void uninstallListeners(JPanel panel) {
        panel.removePropertyChangeListener(this);
    }
    
    @Override
    protected void installDefaults(JPanel panel) {
        this.updateStyle(panel);
    }
    
    @Override
    protected void uninstallDefaults(JPanel panel) {
        SynthContext context = this.getContext(panel, 1);
        this.style.uninstallDefaults(context);
        context.dispose();
        this.style = null;
    }
    
    private void updateStyle(JPanel panel) {
        SynthContext context = this.getContext(panel, 1);
        this.style = SynthLookAndFeel.updateStyle(context, this);
        context.dispose();
    }
    
    @Override
    public SynthContext getContext(JComponent component) {
        return this.getContext(component, this.getComponentState(component));
    }
    
    private SynthContext getContext(JComponent component, int n) {
        return SynthContext.getContext(component, this.style, n);
    }
    
    private int getComponentState(JComponent component) {
        return SynthLookAndFeel.getComponentState(component);
    }
    
    @Override
    public void update(Graphics graphics, JComponent component) {
        SynthContext context = this.getContext(component);
        SynthLookAndFeel.update(context, graphics);
        context.getPainter().paintPanelBackground(context, graphics, 0, 0, component.getWidth(), component.getHeight());
        this.paint(context, graphics);
        context.dispose();
    }
    
    @Override
    public void paint(Graphics graphics, JComponent component) {
        SynthContext context = this.getContext(component);
        this.paint(context, graphics);
        context.dispose();
    }
    
    protected void paint(SynthContext synthContext, Graphics graphics) {
    }
    
    @Override
    public void paintBorder(SynthContext synthContext, Graphics graphics, int n, int n2, int n3, int n4) {
        synthContext.getPainter().paintPanelBorder(synthContext, graphics, n, n2, n3, n4);
    }
    
    @Override
    public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
        if (SynthLookAndFeel.shouldUpdateStyle(propertyChangeEvent)) {
            this.updateStyle((JPanel)propertyChangeEvent.getSource());
        }
    }
}
